/*
 * icon.hh - header for the icon widget
 *
 * This file is part of the source distribution of 
 * Sfm - an icon based X11 desktop program 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 */

#ifndef _ICON_HH
#define _ICON_HH

#include <qwidget.h>
#include <qpixmap.h>
#include <qbitmap.h>
#include <qlist.h>
#include <qstring.h>
#include <qfile.h>
#include <qfileinf.h>
#include <qdir.h>

#include "deskicon.hh"

class QFileInfo ;
class QLineEdit ;

class IconProgram {
  
  int instructions ; // how many instructions to execute
  short int *program ;
  size_t progSize ; // physical allocation
  size_t realSize ; // logical size

public:
  IconProgram (const char *) ;

  enum keywords {
    
    pen_iconcolour , // set colour to the current icon colour
    brush_iconcolour ,
    pen_shadowcolour , // set colour to shadow
    brush_shadowcolour ,
    pen_outlinecolour , // set colour to default outline
    brush_outlinecolour ,
    pen_colour , // set to given colour: expects 3 (u-)integers to follow 
    brush_colour ,
    fillpoly,  /* ( fill a polygon: expects number of vertices then 
		* vertices to follow (all uint) */
    linepoly /* draw polygon outline only */
  } ;

  enum iconState {
    normal,
    selected,
    focus,
    dragdrop,
    emboss,
    bitmap 
  } ;

  void addInstruction (const short int *data, const int n) ;
  void addInstruction (const short int i) ;
  void draw (QPainter &p, iconState); 
  ~IconProgram () { delete[] program ; }
  
} ;

class Icon : public QObject {

  Q_OBJECT ;

  IconProgram *program ; // program to draw this icon
  
  int scale ;
  QRect bound_rect ; // rectangle bounding icon, but not label
  QRect *label_rect ; // bounds label

  // these strings hold lookup-keys to the appropriate
  // pixmaps. Its a bug having this fixed length.
  char pm_normal[256] ;
  char pm_selected[256] ;
  char pm_focus[256] ;
  char pm_drag_drop[256] ;
  char pm_embossed[256] ;
  char mask[256] ;

  QFileInfo *finfo ;

  char *label ;
  char *icon_file ;
  
  bool being_dragged ;
  bool selected ;
  bool drag_drop ;
  bool focus ;
  bool hidden ;
  bool embossed ;
  
  bool accepts_drag_drop ;

  void construct_image (QPixmap *, IconProgram::iconState) ;


public:
  
  Icon (const char *name, const char* zicname=0, const int scale=10, bool file=true) ;
  Icon (const Icon *) ;
  ~Icon () ;

  void draw (QPainter &p) ;
  
  bool is_selected () const { return selected ; }
  bool is_drag_drop () const { return drag_drop ; }
  bool is_hidden () const { return hidden ; }

  void show () { hidden = false ; }
  void hide () { hidden = true ; }
  int width () const { return bound_rect.width() ; }
  int height () const { return bound_rect.height() ; }
  int x() const { return bound_rect.x() ; }
  int y() const { return bound_rect.y() ; }
  const QPoint pos () { return bound_rect.topLeft () ; }

  bool has_file() const { return (finfo == 0 ? false : true ) ; }
  bool is_dir () const { return (finfo == 0 ? false : finfo->isDir()) ; }
  const QDir get_dir () const { return finfo->dir() ; } 

  const QRect &rect () const { return bound_rect ; }
  void move (int x, int y) { bound_rect.moveTopLeft (QPoint(x,y)) ; } 
  void move (QPoint p) { bound_rect.moveTopLeft (p) ; }
  void offset (const QPoint &p) { bound_rect.moveBy (p.x(), p.y()) ; }

  bool has_focus () const { return focus ; }
  void take_focus () { focus = true ; }
  void drop_focus () { focus = false ; }

  void set_accepts_drag_drop (bool s) { accepts_drag_drop = s ; } 
  bool will_accept_drag_drop () const { return accepts_drag_drop ; }

  void set_embossed (bool s) { embossed = s ; }
  bool is_embossed () const { return embossed ; }

  void open_action () ;
  void drag_drop_action (QList<DeskIcon> &) ;

  void draw (Display *, QWidget *) ;
  const char *get_label () const { return label ; }
  const QFileInfo *get_file_info () const { return finfo ; } ;
  // Qt bug? Can't make next line return const char *
  QString get_full_name () const { if (finfo) return finfo->absFilePath() ; else return 0 ; }
  const char *get_icon_file () const { return icon_file ; }
  const QRect &get_label_rect () const { return *label_rect ; } 

  int get_scale () const { return scale ; }
  int label_text_size () const { return scale/8 + 5 ; }
  QBitmap *get_mask() ;
  QPixmap *get_normal() ;
  QPixmap *get_selected () ;
  QPixmap *get_focus() ;
  QPixmap *get_embossed () ;
  QPixmap *get_drag_drop () ;

  enum need_type { zicon = 1 , openaction = 2 , dragdropaction = 3 } ;
  static char *suggest_zic (const char * str) ;
  static void load_rules () ;
  static char *match_to_rules (const char *, need_type) ;
  static char *match_to_rules (QFileInfo *fi, need_type) ;
  static QList<QString> *rule_list ;
  bool is_drag_droppable () const ;

  void new_file_info (QFileInfo &) ;

 public slots:
 void set_selected (bool s) { selected = s ; }
  void toggle_selected () { selected = !selected ; }

  void set_drag_drop (bool s) {if (will_accept_drag_drop()) drag_drop = s ; }
  void rescale (int) ;

  signals:
  void mouse_click (Icon *, QMouseEvent *) ;
  void moved (Icon *) ;
  void open_dir_window (const char *) ;  

} ;

#endif // _ICON_HH
